/* Copyright (c) 1997, MpegTV LLC */
#ifndef _msg_h
#define _msg_h
#ifndef lint
static const char rcsid_msg_h[] = "$Header: /net/hopkins/home/tristan/mpeg/dec/RCS/msg.h,v 1.22 1999/03/01 08:59:46 tristan Exp $";
#endif

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

#include "misc.h"

/*--------------------------------------------------------------------------+
| Configuration																|
+--------------------------------------------------------------------------*/
#define MSG_NEW

#ifdef MSG_NEW

#if 0 /* Tristan Savatier - Fri May 29 19:20:10 PDT 1998 */
//#ifndef MSG_PTR
//#define MSG_PTR			(&Msg)
//#endif
#endif /* 0 - Tristan Savatier - Fri May 29 19:20:10 PDT 1998 */

#endif

/*--------------------------------------------------------------------------+
| Macros																	|
+--------------------------------------------------------------------------*/

/*
 * Print message if specified message type is selected in MSG_PTR->select
 */
#ifdef MSG_DISABLED
#define MSG(type, message)
#else
#define MSG(type, message) STMT( \
	if (MSG_PTR->select & (type)) { \
		_msg_header(MSG_PTR, MSG_HEADER_INFO, __FILE__, __LINE__); \
		printf message; \
		printf("\n"); \
	} \
)
#endif

/*
 * Print message and error diagnotic if specified message type is selected in MSG_PTR->select
 */
#ifdef MSG_DISABLED
#define MSG_ERROR(type, message)
#else
#define MSG_ERROR(type, message) STMT( \
	if (MSG_PTR->select & (MSG_ERR|(type))) { \
		int	_msg_errno = errno; \
		_msg_header(MSG_PTR, MSG_HEADER_ERROR, __FILE__, __LINE__); \
		printf message; \
		printf(": %s\n", msg_error_string(_msg_errno)); \
		errno = _msg_errno; \
	} \
)
#endif

/*
 * Print info message
 */
#define MESSAGE(message) 	MSG(MSG_INFO, message)

/*
 * Print error message
 */
#define MESSAGE_ERROR(message) 	MSG_ERROR(MSG_ERR, message)

/*
 * Print warning message
 */
#ifdef MSG_DISABLED
#define WARNING(printf_param)
#else
#define WARNING(printf_param)	STMT( \
		if (MSG_PTR->select & MSG_WARNING) { \
			_msg_header(MSG_PTR, MSG_HEADER_WARNING, __FILE__, __LINE__); \
			printf printf_param; \
			printf("\n"); \
		} \
	)
#endif


/*
 * Print message then exit
 */
#ifdef MSG_DISABLED
#define FATAL(printf_param)	STMT( \
		exit(EXIT_FAILURE); \
	)
#else
#define FATAL(printf_param)	STMT( \
		_msg_header(MSG_PTR, MSG_HEADER_FATAL, __FILE__, __LINE__); \
		printf printf_param; \
		printf("\n"); \
		exit(EXIT_FAILURE); \
	)
#endif

/*
 * Print message and error diagnotic then exit
 */
#ifdef MSG_DISABLED
#define FATAL_ERROR(printf_param)	STMT( \
		printf("fatal: %s\n", msg_error_string(errno)); \
		exit(EXIT_FAILURE); \
	)
#else
#define FATAL_ERROR(printf_param)	STMT( \
		int	_msg_errno = errno; \
		_msg_header(MSG_PTR, MSG_HEADER_FATAL, __FILE__, __LINE__); \
		printf printf_param; \
		printf(": %s\n", msg_error_string(_msg_errno)); \
		exit(EXIT_FAILURE); \
	)
#endif

/*--------------------------------------------------------------------------+
| Data types																|
+--------------------------------------------------------------------------*/
/* for _msg_header */
typedef enum {
	MSG_HEADER_INFO = 0,	/* for MESSAGE() */
	MSG_HEADER_WARNING = 1,	/* for WARNING() */
	MSG_HEADER_ERROR = 2,	/* for ERROR() i.e. non-fatal errors */
	MSG_HEADER_FATAL = 3	/* for FATAL() i.e. fatal errors */
} MSG_HEADER;

/*
 * Masks used in MSG_PTR->select to select which type of messages should be printed.
 *
 * MSG_PTR->select is a bitwise OR of those flags, and possibly more flags
 * defined by the application.
 */

#define	MSG_INFO		(1 << 0)	/* for MESSAGE() */
#define	MSG_ERR			(1 << 1)	/* for ERROR() i.e. non-fatal errors */
#define	MSG_WARNING		(1 << 2)	/* for WARNING() */

typedef struct msg_struct {
	u_long select;
	const char *progname;
	void (*header_func)(const struct msg_struct *msg_ptr, MSG_HEADER msg_header_type, const char *filename, int line, char *buffer, u_int size);
} MSG_STRUCT;

/*--------------------------------------------------------------------------+
| Externals																	|
+--------------------------------------------------------------------------*/
#ifdef MSG_NEW

#if 0 /* Tristan Savatier - Fri May 29 19:18:49 PDT 1998 */
//extern MSG_STRUCT Msg;
#endif /* 0 - Tristan Savatier - Fri May 29 19:18:49 PDT 1998 */

extern const MSG_STRUCT Msg_dummy;

#else

extern u_long Msg_select;
extern const char *Msg_progname;

#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifndef MSG_DISABLED

extern const char * _msg_basename(const char *);
extern void _msg_header(const MSG_STRUCT *msg_ptr, MSG_HEADER msg_header_type, const char *filename, int line);

const char *msg_error_string(int code);

#endif

#ifndef MSG_NEW

extern const char * (*Msg_header_func)(const MSG_HEADER msg_header_type, const char *filename, int line);

#endif

#ifdef __cplusplus
}
#endif


#endif /* _msg_h */
